@@ -25,6 +25,13 @@
 #include "gtkaccessible.h"
 #include "gtkalias.h"
 
+/**
+ * SECTION:gtkaccessible
+ * @Short_description: Accessibility support for widgets
+ * @Title: GtkAccessible
+ */
+
+
 static void gtk_accessible_real_connect_widget_destroyed (GtkAccessible *accessible);
 
 G_DEFINE_TYPE (GtkAccessible, gtk_accessible, ATK_TYPE_OBJECT)
@@ -40,6 +47,44 @@ gtk_accessible_class_init (GtkAccessibleClass *klass)
   klass->connect_widget_destroyed = gtk_accessible_real_connect_widget_destroyed;
 }
 
+/**
+ * gtk_accessible_set_widget:
+ * @accessible: a #GtkAccessible
+ * @widget: a #GtkWidget
+ *
+ * Sets the #GtkWidget corresponding to the #GtkAccessible.
+ *
+ * Since: 2.22
+ **/
+void
+gtk_accessible_set_widget (GtkAccessible *accessible,
+                           GtkWidget     *widget)
+{
+  g_return_if_fail (GTK_IS_ACCESSIBLE (accessible));
+
+  accessible->widget = widget;
+}
+
+/**
+ * gtk_accessible_get_widget:
+ * @accessible: a #GtkAccessible
+ *
+ * Gets the #GtkWidget corresponding to the #GtkAccessible. The returned widget
+ * does not have a reference added, so you do not need to unref it.
+ *
+ * Returns: (transfer none): pointer to the #GtkWidget corresponding to
+ *   the #GtkAccessible, or %NULL.
+ *
+ * Since: 2.22
+ **/
+GtkWidget*
+gtk_accessible_get_widget (GtkAccessible *accessible)
+{
+  g_return_val_if_fail (GTK_IS_ACCESSIBLE (accessible), NULL);
+
+  return accessible->widget;
+}
+
 /**
  * gtk_accessible_connect_widget_destroyed
  * @accessible: a #GtkAccessible
